<?php
/**
 * video: embeds a video
 *
 * Syntax:
 *   {{video>http://www.yourpage.com/yourmovie.avi [width,height]|Alternate text}}
 *   {{flash>http://www.yoursite.com/yourflash.swf [width,height]|Alternate text}}
 *   etc for each defined type
 *
 * @license    GPL 2 (http://www.gnu.org/licenses/gpl.html)
 * @author     Jason Byrne <jbyrne@floridascale.com>
 * @author     Chris Smith <chris@jalakai.co.uk>
 */
 
if(!defined('DOKU_INC')) define('DOKU_INC',realpath(dirname(__FILE__).'/../../').'/');
if(!defined('DOKU_PLUGIN')) define('DOKU_PLUGIN',DOKU_INC.'lib/plugins/');
require_once(DOKU_PLUGIN.'syntax.php');

if (!function_exists('hsc')) {
  function hsc($string){ return htmlspecialchars($string, ENT_QUOTES, 'UTF-8'); }
}
 
//------------------------[ Settings ] ---------------------------------------------
global $js_ok;
 
// enable/disable javascript urls
$js_ok = false;
 
/**
 * All DokuWiki plugins to extend the parser/rendering mechanism
 * need to inherit from this class
 */
class syntax_plugin_video extends DokuWiki_Syntax_Plugin {
 
    var $settings = array(
      'flash' => array(
         'classid' => 'clsid:D27CDB6E-AE6D-11cf-96B8-444553540000',
         'mime' => 'application/x-shockwave-flash',
         'codebase' => 'http://download.macromedia.com/pub/shockwave/cabs/flash/swflash.cab',
         'plugin' => 'http://www.macromedia.com/go/getflashplayer',
         'param' => array('quality' => 'high'),
         'icon' => 'lib/images/fileicons/avi.png'
         ),
      'midi' => array(
         'classid' => 'clsid:02BF25D5-8C17-4B23-BC80-D3488ABDDC6B',
         'mime' => 'audio/midi',
         'codebase' => 'http://www.apple.com/qtactivex/qtplugin.cab',
         'plugin' => 'http://www.apple.com/quicktime/download/',
         'param' => array(),
         'icon' => 'lib/images/fileicons/avi.png'
         ),
      'movie' => array(
         'classid' => 'clsid:02BF25D5-8C17-4B23-BC80-D3488ABDDC6B',
         'mime' => 'video/quicktime',
         'codebase' => 'http://www.apple.com/qtactivex/qtplugin.cab',
         'plugin' => 'http://www.apple.com/quicktime/download/',
         'param' => array('autoplay' => 'true'),
         'icon' => 'lib/images/fileicons/avi.png'
         ),
      'mp3' => array(
         'classid' => 'clsid:02BF25D5-8C17-4B23-BC80-D3488ABDDC6B',
         'mime' => 'audio/mpeg',
         'codebase' => 'http://www.apple.com/qtactivex/qtplugin.cab',
         'plugin' => 'http://www.apple.com/quicktime/download/',
         'param' => array(),
         'icon' => 'lib/images/fileicons/avi.png'
         ),
      'mpeg' => array(
         'classid' => 'clsid:02BF25D5-8C17-4B23-BC80-D3488ABDDC6B',
         'mime' => 'video/mpeg',
         'codebase' => 'http://www.apple.com/qtactivex/qtplugin.cab',
         'plugin' => 'http://www.apple.com/quicktime/download/',
         'param' => array(),
         'icon' => 'lib/images/fileicons/avi.png'
         ),
      'realvideo' => array(
         'classid' => 'clsid:CFCDAA03-8BE4-11cf-B84B-0020AFBBCCFA',
         'mime' => 'video/vnd.rn-realvideo',
         'codebase' => '',
         'plugin' => '',
         'param' => array('autostart' => 'true'),
         'icon' => 'lib/images/fileicons/avi.png'
         ),
      'video' => array(
         'classid' => 'CLSID:05589FA1-C356-11CE-BF01-00AA0055595A',
         'mime' => 'application/x-mplayer2',
         'codebase' => '',
         'plugin' => '',
         'param' => array('autostart' => 'true'),
         'icon' => 'lib/images/fileicons/avi.png'
         ),
    );
 
    function getInfo(){
        return array(
            'author' => 'Jason Byrne',
            'email'  => 'jbyrne@floridascale.com',
            'date'   => '2005-10-18',
            'name'   => 'video',
            'desc'   => 'Embed media content into a wiki page',
            'url'    => 'http://wiki.splitbrain.org/plugin:video',
        );
    }
 
    function getType() { 
        return 'substition'; 
    }
    
    function getSort(){
        return 306;
    }
    
    function connectTo($mode) { 
 
        $types = array_keys($this->settings);
        
        foreach ($types as $type)
          $this->Lexer->addSpecialPattern('{{'.$type.'>.*?}}',$mode,'plugin_video'); 
    }
    
    function handle($match, $state, $pos, &$handler){
      list($type, $data) = explode('>',substr($match, 2,-2),2);
      $data = html_entity_decode($data);
      list($url, $alt) = explode('|',$data,2);
			$matches=array();
      if (preg_match('/(.*)\[(.*)\]$/',trim($url),$matches)) {
        $url = $matches[1];
        if (strpos($matches[2],',') !== false) {
          list($w, $h) = explode(',',$matches[2],2);
        } else {
          $h = $matches[2];
          $w = '640';
        }
      } else {
        $w = '640';
        $h = '480';
      }
      
      if (!isset($alt)) $alt = $type.' file: '.$url;
      if (!$js_ok && substr($url,0,11) == 'javascript:') $url = 'error';
 
      return array(trim($type), hsc(trim($url)), hsc(trim($alt)), hsc(trim($w)), hsc(trim($h))); 
    }
    
    function render($mode, &$renderer, $data) {
 
      list($type, $url, $alt, $w, $h) = $data;
      $config = $this->settings[$type];
      
      if($mode == 'xhtml'){
        if ($url != 'error') {
 
          $param = '';
          $mime = '';
					$src = '';
          $classid = '';
          $codebase = '';
          $icon = '';
 
          if ($config['mime'])  $mime = ' type="'.$config['mime'].'"';
#         if ($config['plugin']) $src = ' pluginspage="'.$config['plugin'].'"';
          if ($config['plugin']) $home = ' data="'.$config['plugin'].'"';
          if ($config['classid']) $classid .= ' classid="'.$config['classid'].'"';
          if ($config['codebase']) $codebase .= ' codebase="'.$config['codebase'].'"';
          if ($config['icon']) $icon .= '<img src="'.DOKU_BASE.$config['icon'].'" alt="" />';
 
          foreach ($config['param'] as $name => $value) {
            $param .= "  <param name=\"$name\" value=\"$value\">\n";
            $attr .= " $name=\"$value\"";
          }
/*
<object type='{$mime}' data='{$url}' width='{$w}' height='{$h}'>
	<param name='{name}' value='{$url}' />
	<img src='{$url_none}' width='{$w}' height='{$h}' alt='' />
</object>
*/


          $renderer->doc .= "<object $classid $codebase class=\"plugin_video\" width=\"$w\" height=\"$h\" >\n";
          $renderer->doc .= "  <param name=\"src\" value=\"$url\">\n";
          $renderer->doc .= $param;

          // use IE conditional comments to conditionally exclude the following content from IE LTE 6					
					$renderer->doc .= "<!--[if gt IE 6]>-->\n";
/*
          $renderer->doc .= "  <embed $mime $src class=\"plugin_video\" src=\"$url\" width=\"$w\" height=\"$h\" controller=\"true\">\n";
          $renderer->doc .= "    <div><img src=\"lib/images/fileicons/avi.png\" alt=\"\" /><a href=\"$url\">$alt</a></div>\n";
          $renderer->doc .= "  </embed>\n";
*/
            
					$renderer->doc .= "<object $mime $src data='{$url}' width='{$w}' height='{$h}'>\n";
					$renderer->doc .= $param;
					$renderer->doc .= "	<img src='{$url_none}' width='{$w}' height='{$h}' alt='' />\n";
					$renderer->doc .= "</object>\n";
		
          // close the IE conditional exclude
					$renderer->doc .= "<!--<![endif]-->\n";
					
          $renderer->doc .= "</object>\n";
        } else {
          $renderer->doc .= "<div>$icon<a href=\"$url\">$alt</a></div>\n";
        }
        return true;
      }
      return false;
    }
}
 
?>